import { Module } from '@nestjs/common';
import { HttpModule } from '@nestjs/axios';
import { GraphQLModule } from '@nestjs/graphql';
import { ApolloDriver, ApolloDriverConfig } from '@nestjs/apollo';
import { upperDirectiveTransformer } from './common/lib/upper-case.directive';
import { HealthController } from '@apis/other/health.controller';
import { GrpcClient } from '@grpc.ts/nestjs-client';
import { join } from 'path';
import * as GraphQLJSON from 'graphql-type-json';
import { ClsModule } from 'nestjs-cls';
import LogHelp from '@dt/log-help';
import { v4 as uuidv4 } from 'uuid';
import { AllExceptionsFilter } from '@common/filter/all-exceptions.filter';
import { LoggerService } from '@common/logger/logger.service';
import { LoggingInterceptor } from '@common/logger/logging.interceptor';
import { StringOrIntScalar } from '@common/scalar/string-or-int.scalar';
import { LoggerModule } from '@common/logger/logger.module';
import { ENV_NAME_ENUM } from '@common/lib/enum';
import { QyWechatbotModule, QyWechatbotService } from '@dt/nestjs-qywechatbot';
import { WalletModule } from './wallet/wallet.module';
import { AssetWorkflowModule } from './assetworkflow/assetworkflow.module';

function getUserInfo(headers: any) {
  const jwtUserStr = headers['x-user'];
  if (jwtUserStr) {
    try {
      return JSON.parse(jwtUserStr);
    } catch (err) {
      console.error('jwtUserStr JSON.parse error', err);
    }
  }
  return null;
}

@Module({
  imports: [
    // GrpcClient.register([
    // ]),
    GraphQLModule.forRootAsync<ApolloDriverConfig>({
      driver: ApolloDriver,
      imports: [
        LoggerModule,
        QyWechatbotModule.forRoot({ key: process.env.QYWX_BOT_KEY }),
      ],
      inject: [LoggerService, QyWechatbotService],
      useFactory: async (
        logger: LoggerService,
        qyWechatBot: QyWechatbotService,
      ) => {
        const isDebug = process.env.ENV_NAME !== ENV_NAME_ENUM.PORDUCT;
        return {
          typePaths: ['./**/*.graphql'],
          transformSchema: (schema) =>
            upperDirectiveTransformer(schema, 'upper'),
          installSubscriptionHandlers: true,
          buildSchemaOptions: {},
          context: (req) => {
            const { body, raw, method, headers = {} } = req;
            const url = raw.originalUrl || raw.url;
            logger.dtlog.setCategory('web');
            logger.dtlog.set({ url });
            logger.log({ method, body, headers });
            logger.dtlog.setCategory('');
            req.user = getUserInfo(headers);
          },
          formatError: (error: any) => {
            logger.error(error);
            qyWechatBot.sendMarkdownMessage(
              `** HQ Graphql 错误报告**
** 消息**: ${error?.message}
** 路径**: ${error?.path}
** 错误码**: ${error?.extensions?.code}
** Trace Id**: ${logger.dtlog.get('trace_id')}
** 环境**: ${process.env?.RABBIT_VHOST}
              `,
            );
            if (!isDebug) {
              delete error.locations;
              delete error.extensions;
            }
            return error;
          },
        };
      },
    }),
    HttpModule.register({ timeout: 5000 }),
    ClsModule.forRoot({
      global: true,
      middleware: {
        // automatically mount the
        // ClsMiddleware for all routes
        mount: true,
        // and use the setup method to
        // provide default store values.
        setup: (cls, req) => {
          cls.set('url', req.headers['url']);
          const dtlog = new LogHelp();
          dtlog.set({ url: cls.get('url') });
          let traceId = req.headers['x-b3-traceid'] || req.headers['trace_id'];

          if (!traceId) {
            traceId = uuidv4().replace(/-/g, '');
            req.headers['trace_id'] = traceId;
          }
          traceId && dtlog.set({ trace_id: traceId });
          cls.set('tsf-system-tags', req.headers['tsf-system-tags'] || '');
          cls.set('tsf-mesh-tag', req.headers['tsf-system-tags'] || '');
          cls.set('dtlog', dtlog);
        },
      },
    }),
    QyWechatbotModule.forRoot({
      key: process.env.QYWX_BOT_KEY,
    }),
    WalletModule,
    AssetWorkflowModule,
  ],
  controllers: [HealthController],
  providers: [
    HttpModule,
    StringOrIntScalar,
    LoggingInterceptor,
    AllExceptionsFilter,
    LoggerService,
  ],
})
export class AppModule {}
