import { Metadata } from '@grpc/grpc-js';
import { lastValueFrom } from 'rxjs';
import { ClientGrpc } from '@nestjs/microservices';
import {CommonResponse} from "@common/interfaces/response.interface";


export abstract class BaseGrpcService<T = any> {
  protected serviceClient: T;

  protected setServiceClient(client: ClientGrpc, serviceName: string) {
    this.serviceClient = client.getService<T>(serviceName);
  }

  /**
   * 调用 grpc
   * @param method
   * @param input
   * @param context
   * @protected
   */
  public async callGrpc<R = any>(
    method: string,
    input: any,
    context: any,
  ): Promise<R> {
    const metadata = this.createMetadata(context);
    this.processInput(input, context);
    const resp = await lastValueFrom<CommonResponse<R>>(
      this.serviceClient[method](input, metadata),
    );
    // 检查响应是否存在
    if (!resp) {
      throw new Error('GRPC response is undefined');
    }

    // 处理 Go 的 int32 类型
    const code = typeof resp.code === 'number' ? resp.code :
      (resp as any).Code || (resp as any).code || 0;

    if (code !== 0) {
      const msg = resp.msg || (resp as any).Msg || (resp as any).message || 'Unknown error';
      throw new Error(msg);
    }
    return resp.data;
  }

  /**
   * 处理入参
   * @param input
   * @param context
   * @private
   */
  private processInput(input: any, context: any) {
    if (context?.req?.user) {
      input.userId = context.req.user.userId;
    }
  }

  /**
   * 创建 metadata
   * @param context
   * @private
   */
  private createMetadata(context: any): Metadata {
    const metadata = new Metadata();
    if (context == null) {
      return metadata;
    }
    const headers = context.req.headers;

    // 从请求头获取 trace_id
    if (headers.trace_id) {
      metadata.add('trace_id', headers.trace_id);
    }

    // 添加其他 headers
    metadata.add('x-version', headers['x-version'] || 'v1');
    metadata.add('hq-version', headers['hq-version'] || '1.0.0');
    metadata.add('tsf-system-tags', headers['tsf-system-tags'] || '');
    metadata.add('tsf-mesh-tag', headers['tsf-mesh-tag'] || '');
    metadata.add('userId', context.req.user?.userId || '');

    return metadata;
  }
}
